﻿using System;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Windows.Controls;
using Telerik.Windows.Data;

namespace SampleBinding_SL
{
	public partial class MainPage : UserControl
	{
		//private readonly int testSize = 100;
		private readonly int testSize = 1;
		private readonly int loadSize = 100;
		private readonly VirtualQueryableCollectionView itemsSource;

		public MainPage()
		{
			InitializeComponent();

			this.itemsSource = this.GetItemsSource();
			this.xTreeView.ItemsSource = this.itemsSource;
		}

		private VirtualQueryableCollectionView GetItemsSource()
		{
			var items = new VM();
			items.Clear();
			var itemsSource = new VirtualQueryableCollectionView(items) { LoadSize = loadSize, VirtualItemCount = testSize };
			itemsSource.ItemsLoading += (s, e) =>
			{
				var newItems = new VM();
				newItems.Clear();
				//for (int count = e.StartIndex; count < e.StartIndex + e.ItemCount; ++count)
				for (int count = e.StartIndex; count < e.StartIndex + 15; ++count)
				{
					newItems.Add(new BI() { Name = count.ToString() });
				}
				itemsSource.Load(e.StartIndex, newItems);
				itemsSource.VirtualItemCount = e.StartIndex + newItems.Count;
			};

			return itemsSource;
		}
	}

	public class VM : ObservableCollection<BI>
	{
		public VM()
		{
			for (int i = 0; i < 10; i++)
			{
				BI bi = new BI()
				{
					Name = i.ToString(),
					Level = 1,
					IsExpanded = false,
					IsSelected = false
				};

				for (int j = 0; j < 3; j++)
				{
					BI bi2 = new BI()
					{
						Name = bi.Name + "." + j,
						Level = 2,
						IsExpanded = false,
						IsSelected = false
					};

					for (int k = 0; k < 3; k++)
					{
						BI bi3 = new BI()
						{
							Name = bi2.Name + "." + k,
							Level = 3,
							IsExpanded = false,
							IsSelected = false
						};
						bi2.Items.Add(bi3);
					}

					bi.Items.Add(bi2);
				}

				this.Add(bi);
			}
		}
	}

	public class BI : INotifyPropertyChanged
	{
		public override string ToString()
		{
			return this.Name + " @ " + this.Level;
		}

		public BI()
		{
			this.Items = new ObservableCollection<BI>();
		}
		public ObservableCollection<BI> Items { get; set; }
		public string Name { get; set; }
		public int Level { get; set; }

		private bool isExpanded;
		public bool IsExpanded
		{
			get
			{
				return this.isExpanded;
			}
			set
			{
				if (value != isExpanded)
				{
					this.isExpanded = value;
					this.OnPropertyChanged("IsExpanded");
				}
			}
		}

		private bool isSelected;
		public bool IsSelected
		{
			get
			{
				return this.isSelected;
			}
			set
			{
				if (value != this.isSelected)
				{
					this.isSelected = value;
					this.OnPropertyChanged("IsSelected");
				}
			}
		}

		public event PropertyChangedEventHandler PropertyChanged;

		protected virtual void OnPropertyChanged(String propertyName)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
			}
		}

	}
}
